import { CommandRegistry } from '@lumino/commands';
import { Menu, MenuBar } from '@lumino/widgets';
import { FileMenu } from './file';
import { EditMenu } from './edit';
import { HelpMenu } from './help';
import { KernelMenu } from './kernel';
import { RunMenu } from './run';
import { SettingsMenu } from './settings';
import { ViewMenu } from './view';
import { TabsMenu } from './tabs';
import { IMainMenu } from './tokens';
/**
 * The main menu class.  It is intended to be used as a singleton.
 */
export declare class MainMenu extends MenuBar implements IMainMenu {
    /**
     * Construct the main menu bar.
     */
    constructor(commands: CommandRegistry);
    /**
     * The application "Edit" menu.
     */
    readonly editMenu: EditMenu;
    /**
     * The application "File" menu.
     */
    readonly fileMenu: FileMenu;
    /**
     * The application "Help" menu.
     */
    readonly helpMenu: HelpMenu;
    /**
     * The application "Kernel" menu.
     */
    readonly kernelMenu: KernelMenu;
    /**
     * The application "Run" menu.
     */
    readonly runMenu: RunMenu;
    /**
     * The application "Settings" menu.
     */
    readonly settingsMenu: SettingsMenu;
    /**
     * The application "View" menu.
     */
    readonly viewMenu: ViewMenu;
    /**
     * The application "Tabs" menu.
     */
    readonly tabsMenu: TabsMenu;
    /**
     * Add a new menu to the main menu bar.
     */
    addMenu(menu: Menu, options?: IMainMenu.IAddOptions): void;
    /**
     * Dispose of the resources held by the menu bar.
     */
    dispose(): void;
    /**
     * Handle the disposal of a menu.
     */
    private _onMenuDisposed;
    private _items;
}
